#ifndef MALACHSCRIPT_BOUNDNAMESPACE_HPP
#define MALACHSCRIPT_BOUNDNAMESPACE_HPP

#include <optional>
#include <unordered_map>
#include "../CoreData/Identifier.hpp"
#include "BoundType.hpp"
#include "BoundVariable.hpp"

namespace MalachScript::Binder {
    class BoundNamespace {
    public:
        inline void RegisterNamespace(const Identifier& identifier, BoundNamespace* ns) {
            _namespaces[identifier] = ns;
        }

        inline void RegisterType(const Identifier& identifier, BoundType* type) { _types[identifier] = type; }
        inline void RegisterType(const Identifier& identifier, const BoundType* type) {
            _types[identifier] = const_cast<BoundType*>(type);
        }
        inline void AddVariable(const Identifier& identifier, BoundVariable* var) { _variables[identifier] = var; }

        inline std::optional<BoundType*> ResolveType(const Identifier& identifier) const noexcept {
            auto find = _types.find(identifier);
            if (find == _types.end()) {
                return {};
            }
            return find->second;
        }
        inline std::optional<BoundNamespace*> ResolveNamespace(const Identifier& identifier) const noexcept {
            auto find = _namespaces.find(identifier);
            if (find == _namespaces.end()) {
                return {};
            }
            return find->second;
        }

        inline const std::unordered_map<Identifier, BoundNamespace*>& GetNamespaces() const noexcept {
            return _namespaces;
        }
        inline const std::unordered_map<Identifier, BoundType*>& GetTypes() const noexcept { return _types; }

    private:
        std::unordered_map<Identifier, BoundNamespace*> _namespaces;
        std::unordered_map<Identifier, BoundType*> _types;
        std::unordered_map<Identifier, BoundVariable*> _variables;
    };
}

#endif // MALACHSCRIPT_BOUNDNAMESPACE_HPP
